/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: %W% %G% */
/*************************************************************************
 * FILE:      sendGPIBCommand.c
 * Author:    Claude Saunders
 * 
 * Purpose:   Program for sending an arbitrary GPIB command
 *            to an instrument. This is a tricky business given
 *            EPICS and the GPIB support library, so use it with
 *            caution. 
 *************************************************************************/

#include <stdio.h>
#include <string.h>
#include "cadef.h"
#include "ezca.h"
#include "SDDS.h"          /* SDDS library header */

#define  VERSION   "V1.0"
#define  GPIB_PORT_CMD_LENGTH 14900

/* Function Prototypes */
int process_cmdline_args(int argc, char *argv[], char *gpibport, unsigned char *cmd);
void usage(char *name);

int main(int argc, char *argv[]) {
  char gpibport[255];           /* name of gpib port */
  unsigned char cmd[GPIB_PORT_CMD_LENGTH+3];     
  char resp[GPIB_PORT_CMD_LENGTH+3]; 
  char writepv[255];
  char readpv[255];
  char procpv[255];
  char nordpv[255];
  long nord;
  short one = 1;
  int status;                  /* status of ezca operations */

  process_cmdline_args(argc, argv, gpibport, cmd);
  
  /* Note: we are limited here by the waveform nelm and CA buffer size */
  if (strlen((char*)cmd) > GPIB_PORT_CMD_LENGTH) {
    fprintf(stderr,"%s: Command/Query length limited to %d chars\n",
	    argv[0], GPIB_PORT_CMD_LENGTH);
    return(1);
  }

  /* Form proper process variable name */
  strcpy(writepv,gpibport);
  strcat(writepv,":sendCmdWF");

  strcpy(readpv,gpibport);
  strcat(readpv,":getRespWF");

  strcpy(procpv,gpibport);
  strcat(procpv,":getRespWF.PROC");

  strcpy(nordpv,gpibport);
  strcat(nordpv,":getRespWF.NORD");

  ezcaSetTimeout(0.1);
  ezcaSetRetryCount(100);

  strcat((char*)cmd,"\n");
  /* Write scope command string */
  status = ezcaPut(writepv, ezcaByte, strlen((char*)cmd)+1, (unsigned char *)cmd);
  if (status) {
    fprintf(stderr, "%s: Unable to find desired gpib port\n",argv[0]);
    ca_task_exit();
    return(1);
  }

  if (strchr((char*)cmd, '?')) {
    /* Process the record to get the query response */
    status = ezcaPut(procpv, ezcaShort, 1, &one);
    if (status) {
      fprintf(stderr, "%s: Unable to process response record\n",argv[0]);
      ca_task_exit();
      return(1);
    }
    status = ezcaGet(nordpv, ezcaLong, 1, &nord);
    if (status) {
      fprintf(stderr, "%s: Unable to retrieve response length\n",argv[0]);
      ca_task_exit();
      return(1);
    }
    /* Read the query response */
    status = ezcaGet(readpv, ezcaByte, nord, &resp);
    if (status) {
      fprintf(stderr, "%s: Unable to retrieve response\n",argv[0]);
      ca_task_exit();
      return(1);
    }
    resp[GPIB_PORT_CMD_LENGTH] = '\0';
    fprintf(stdout,"%s",resp);
  }
  ca_task_exit();
  return(0);
}
/*************************************************************************
 * FUNCTION : process_cmdline_args()
 * PURPOSE  : Get command line arguments and set flags appropriately.
 * ARGS in  : argc, argv - standard unix
 * ARGS out : gpibport - name of scope
 *            cmd - command/query string to be sent to GPIB instrument
 * GLOBAL   : nothing
 * RETURNS  : 0
 ************************************************************************/  
int
process_cmdline_args(int argc, char *argv[], char *gpibport, unsigned char *cmd)
{
  int cmderror = 0;

  *gpibport = '\0';
  *cmd = '\0';

  /* Process command line options */
  if (argc != 3)
    cmderror++;
  else {
    strcpy(gpibport,argv[1]);
    strcpy((char*)cmd,argv[2]);
    }

  if (cmderror) {
    usage(argv[0]);
    exit(1);
  }
  return(0);
}

/*************************************************************************
 * FUNCTION : usage()
 * PURPOSE  : Describes command line options available to client.
 * ARGS in  : char *name - name of executable program
 * ARGS out : none
 * GLOBAL   : nothing
 * RETURNS  : none
 ************************************************************************/  
void usage(char *name)
{
  fprintf(stderr,"Send GPIB Command Utility %s\n",VERSION);
  fprintf(stderr,"-------------------------------\n");
  fprintf(stderr,"usage: %s <gpib-portname> <GPIB-command>\n",name);
  fprintf(stderr,"purpose:\tWrites <GPIB-command> to <gpib-portname>.\n");
  fprintf(stderr,"\t\tAny response is written to stdout. It is assumed\n");
  fprintf(stderr,"\t\tthat all queries contain the ? character.\n");
}

